def realwork(vars, moddefns, target):
    version_suffix = "%r%r" % sys.version_info[:2]
    print "# Makefile for Microsoft Visual C++ generated by freeze.py script"
    print
    print 'target = %s' % target