# Defaults for generic-dh-make-2008 initscript
# sourced by /etc/init.d/generic-dh-make-2008
# installed at /etc/default/generic-dh-make-2008 by the maintainer scripts

#
# This is a POSIX shell fragment
#

# Additional options that are passed to the Daemon.
DAEMON_OPTS=""
