/*
 * Acme Corporation and its licensors retain all intellectual property and
 * proprietary rights in and to this software and related documentation.
 * Any use, reproduction, disclosure, or distribution of this software
 * and related documentation without an express license agreement from
 * Acme Corporation is strictly prohibited.
 *
 * Please refer to the applicable Acme end user license agreement (EULA)
 * associated with this source code for terms and conditions that govern
 * your use of this Acme software.
 *
 */
